# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/00_core.ipynb.

# %% auto 0
__all__ = ['Spec', 'matr', 'ImportedAttribute', 'iattr', 'matrs', 'iattrs', 'pmod']

# %% ../nbs/00_core.ipynb 5
from functools import wraps

# %% ../nbs/00_core.ipynb 7
from types import ModuleType
from typing import (Self, Type, Union, Optional, NamedTuple, ClassVar, Iterable, )

# %% ../nbs/00_core.ipynb 9
from nchr import U1
from nlit import (
    ASSET, AREPR, IMPORTED, DEFAULT, MODULE, 
     __DOC__, __MODULE__, __QUALNAME__, __ANNOTATIONS__
)
from pdec import slotprops, Property
from itry import imod, T, Types, Namespace, ImportedAsset

# %% ../nbs/00_core.ipynb 11
_AREPR, _ASSET, _IMPORTED, _MODULE = f'{U1}{AREPR}', f'{U1}{ASSET}', f'{U1}{IMPORTED}', f'{U1}{MODULE}'

# %% ../nbs/00_core.ipynb 13
class Spec(NamedTuple):
    '''Specification information to import an asset.

    Parameters
    ----------
    name : str, optional
        The name of the module to import from, defaults to None.
        
    attr : str, optional
        The name of the attribute to import from the module, defaults to None.
        
    bases : Union[Types, type]
        Base classes for creating a new type if import fails and default is None.
        
    namespace : dict
        A namespace dictionary for creating a new type.
        
    default : type, optional
        A default value to return if import fails.
    '''
    name: Optional[str] = None
    attr: Optional[str] = None
    bases: Union[Types, type] = tuple()
    namespace: Namespace = dict()
    default: Optional[T] = None
    
    def asdict(self) -> dict:
        return self._asdict()
    
    def getvals(self, obj: object, __type: bool = False, **kwargs) -> dict:
        dct, res = self.asdict(), dict()
        itm = type(obj) if __type else obj
        for k, v in dct.items():
            val = kwargs.get(k, getattr(itm, k, v)) or v
            if val is not None and val:
                res[k] = val
            
        return res
    
    def setvals(self, obj: object, __type: bool = False, **kwargs):
        item = type(obj) if __type else obj
        kwargs.update(dict(__type = __type))
        vals = self.getvals(obj, **kwargs)
        for k, v in vals.items():
            setattr(item, k, v)
        return item

# %% ../nbs/00_core.ipynb 15
@slotprops(dict(
    asset = Property(ASSET, ImportedAsset, doc = 'The imported asset', gen = lambda self: self.__import__()),
    arepr = Property(AREPR, ImportedAsset, doc = 'String representation of the imported asset.', gen = lambda self: self.__asset_repr__()),
    imported = Property(IMPORTED, bool, False, 'Indicates if the import was successful.'),
))
class matr:
    '''A class representing a module's attribute (matr), imported if needed.

    Attributes
    ----------
    name : str
        The name of the module or attribute.
        
    attr : str, optional
        The specific attribute to import, if any.
        
    bases : tuple[type, ...]
        Base classes for creating new types if import fails.
        
    namespace : namespace | dict
        A namespace dictionary for creating new types.
        
    default : Any, optional
        Default value if the import fails.
        
    asset : Union[Type, FunctionType, BuiltinFunctionType, BuiltinMethodType, ModuleType], optional
        Property for the imported asset.
        
    arepr : str, optional
        Property for the string representation of the imported asset.
        
    imported : bool, default = False
        Property indicating if the import was successful.

    Methods
    -------
    io(ins: 'matr')
        Import an instance of matr class.
        
    ib(bases: Types)
        Import bases using the `io` method.
        
    __init__(self, name, attr, bases, namespace, default, eager, **kwargs)
        Initialize the matr instance.
        
    __import__(self)
        Import the asset using the `imod` method.
        
    __asset_repr__(self)
        Return the string representation of the asset.
        
    __call__(self, *args, **kwargs)
        Call the imported asset if callable.
        
    __repr__(self)
        Return the string representation of the matr instance.
        
    __getitem__(self, key)
        Get an item from the imported asset.
        
    __getattribute__(self, __name)
        Override getattribute to prioritize the asset's attributes.
    '''
    name: ClassVar[str]
    attr: ClassVar[Optional[str]] = None
    bases: ClassVar[Union[Types, type]] = tuple()
    namespace: ClassVar[Namespace] = dict()
    default: ClassVar[Optional[T]] = None
    
    asset: ImportedAsset
    arepr: Optional[str]
    imported: bool
    
    @classmethod
    def io(cls: Type[Self], ins: 'matr') -> ImportedAsset:
        '''Import instance of `matr` class'''
        if not isinstance(ins, cls): 
            return ins
        if not isinstance(ins, matr): 
            return ins
        return ins.__import__()
    
    @classmethod
    def ib(cls: Type[Self], bases: Types = tuple()) -> Types:
        '''Import bases using `io` method which is fault tolerant'''
        if not isinstance(bases, (list, tuple, Iterable)): 
            bases = (bases,)
        return tuple(cls.io(base) for base in bases)
    
    def __init__(
        self: Self, 
        name: str, 
        attr: Optional[str] = None, 
        bases: Types = tuple(), 
        namespace: Namespace = dict(), 
        default: Optional[T] = None, 
        eager: bool = False, 
        **kwargs
    ):
        spec = Spec(name, attr, bases, namespace, default)
        spec.setvals(self, __type = False, **kwargs)
        self._imported = False
        if eager:
            self.__import__()
            
    def __import__(self: Self) -> ImportedAsset:
        '''Import asset using `itry.imod` method which is fault tolerant. 
        
        If any of the classes in `self.bases` are `matr` instances,
        they will be imported first and then passed to `imod` as bases.
        '''
        if getattr(self, _ASSET, None) is not None:
            return self.asset
        
        if getattr(self, _IMPORTED, False):
            return self.asset
        
        asset = imod(self.name, self.attr, self.ib(self.bases), self.namespace, self.default)
        self._asset = asset

        
        if getattr(self, _ASSET, None) != getattr(self, DEFAULT, None): 
            self._imported = True
            
        return self.asset
        return self._asset
    
    def __asset_repr__(self: Self) -> Optional[str]:
        '''Return repr of asset if it exists, otherwise return None.'''
        if not hasattr(self, _AREPR): 
            return None
        
        if getattr(self, _AREPR, None) is None: 
            return repr(self._asset)
        
        return self._arepr
    
    def __call__(self: Self, *args, **kwargs) -> ImportedAsset:
        try:
            
            if callable(self.asset): 
                return self.asset(*args, **kwargs)
        except: 
            ...
        return self.asset
        
    def __repr__(self) -> str:
        return self.arepr or f'{self.name}({self.attr})'
    
    def __getitem__(self: Self, key: str) -> ImportedAsset:
        if hasattr(self, _ASSET): 
            return getattr(self._asset, key)
        
        return getattr(self, key, self.default)
    
    def __getattribute__(self: Self, __name: str, __recursed: bool = False):
        # prioritize self first
        try: 
            return super().__getattribute__(__name)
        except AttributeError as err:
            if __recursed:
                raise err
        # then try asset
        try:
            asset = object.__getattribute__(self, _ASSET)
            return object.__getattribute__(asset, __name)
        
        except AttributeError:
            ...
        # then back to self and raise error if not found
        return self.__getattribute__(__name, True)
        return super().__getattribute__(__name)
    
    
# Alias for `matr` class
@wraps(matr, updated=())
class ImportedAttribute(matr):
    '''An alias for the matr class.'''

@wraps(matr, updated=())
class iattr(matr):
    '''An alias for the matr class.'''

# %% ../nbs/00_core.ipynb 22
def matrs(
    name: str, 
    *attrs: str, 
    bases: Union[Types, type] = tuple(), 
    namespace: Namespace = dict(), 
    **kwargs
) -> tuple[matr, ...]:
    '''Try to import multiple attributes from a module, returning a tuple of results.

    Parameters
    ----------
    name : str
        The name of the module to import from.

    attrs : str
        Variable number of attribute names to import from the module.

    bases : Union[Tuple[type, ...], type]
        Base classes for creating new types if import fails.

    namespace : dict
        A namespace dictionary for creating new types.

    **kwargs
        Additional keyword arguments passed to imod.

    Returns
    -------
    Tuple[ImportedAttribute, ...]
        A tuple of the imported attributes or default values.
    '''
    return tuple(matr(name, attr, bases, namespace, **kwargs) for attr in attrs)


@wraps(matrs)
def iattrs(*args, **kwargs):
    return matrs(*args, **kwargs)

# %% ../nbs/00_core.ipynb 24
class pmod(ModuleType):
    '''A proxy for a module (pmod), which dynamically imports and 
    exposes attributes from the specified module.'''
    def __init__(self, module):
        super().__init__(module.__name__, module.__doc__)
        self._module = module

    def __getattr__(self, name: str):
        attr = getattr(self._module, name)
        for dund in (__DOC__, __MODULE__, __QUALNAME__, __ANNOTATIONS__):
            try: setattr(attr, dund, getattr(attr, dund, None))
            except Exception: ...
        return attr

    def __setattr__(self, name: str, value):
        if name == _MODULE:
            super().__setattr__(name, value)
        else:
            setattr(self._module, name, value)

    def __dir__(self):
        return dir(self._module)
        
    def names(self, withmagic: bool = False) -> list[str]:
        return [
            name for name in dir(self) 
            if (True if withmagic else not name.startswith(U1))
        ]
        
    def attrs(
        self: Self, 
        withmagic: bool = False
    ) -> dict:
        vals = {name: getattr(self, name) for name in self.names(withmagic)}
        return vals
    
    def populate(
        self: Self, 
        namespace: Optional[Namespace] = globals(),
        withmagic: bool = False
    ) -> Self:
        namespace = namespace or globals()
        namespace.update(self.attrs(withmagic=withmagic))
        return self
    
    



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/01_inst.ipynb.

# %% auto 0
__all__ = ['moduleloader', 'modulespec', 'pathlike', 'path', 'posixpath', 'sig', 'param', 'logger', 'ichain', 'iprod', 'ziplong',
           'dict_keys', 'dict_values', 'dict_items', 'moduletype', 'wrappertype', 'builtinfunctype', 'trackbacktype',
           'functiontype', 'nonetype', 'mapping', 'iterable', 'nan', 'npnan', 'npNaN', 'npNAN', 'pdna', 'ptnan',
           'number', 'npbool', 'repattern', 'strtemplate', 'nptype', 'npmatrix', 'npdatasource', 'nparray', 'recarray',
           'npempty', 'sparray', 'spmatrix', 'splinalg', 'speigs', 'sppdist', 'cscarray', 'cscmatrix', 'cooarray',
           'coomatrix', 'csrarray', 'csrmatrix', 'diaarray', 'diamatrix', 'dokarray', 'dokmatrix', 'lilarray',
           'lilmatrix', 'bsrarray', 'bsrmatrix', 'pdindex', 'pdseries', 'pdcatindex', 'pddataframe', 'pddelegate',
           'pdcataccessor', 'ptsize', 'pttype', 'ptdevice', 'ptmodule', 'ptdataset', 'ptdataloader', 'pttensorbase',
           'pttensor', 'ptinttensor', 'ptbooltensor', 'ptfloattensor', 'ptmemfmt', 'ptpresfmt', 'ptdatasets',
           'pttransforms', 'ptfunctional', 'plmodule', 'pldatamodule', 'odefn', 'sdefn', 'neurode', 'neursde',
           'augmenter', 'ito', 'brownian', 'anndata', 'gtbase', 'gtdata', 'bgraph', 'dgraph', 'lgraph', 'kgraph',
           'tgraph', 'mgraph', 'klgraph', 'mlgraph', 'tlgraph', 'vidcap', 'image', 'mpl', 'plt', 'artist', 'colormap',
           'lscolormap', 'mplpatch', 'mplaxbase', 'mplaxes', 'sns', 'skestimator', 'skpairwise', 'PHATE', 'richtree',
           'richtext', 'richcons', 'richprog', 'richtask', 'tqdmauto', 'typer']

# %% ../nbs/01_inst.ipynb 5
from functools import wraps

# %% ../nbs/01_inst.ipynb 7
from types import ModuleType
from typing import (Self, Type, Union, Optional, NamedTuple, ClassVar, Iterable, )

# %% ../nbs/01_inst.ipynb 11
from .core import matr

# %% ../nbs/01_inst.ipynb 14
moduleloader = matr('importlib', 'Loader')
'''`importlib.Loader`''';

modulespec = matr('inspect.machinery', 'ModuleSpec')
'''`importlib.machinery.ModuleSpec`''';

# %% ../nbs/01_inst.ipynb 16
pathlike = matr('os', 'PathLike')
'''`os.PathLike`''';

path = matr('pathlib', 'Path')
'''`pathlib.Path`''';

posixpath = matr('pathlib', 'PosixPath')
'''`pathlib.PosixPath`''';

# %% ../nbs/01_inst.ipynb 18
sig = matr('inspect', 'Signature')
'''`inspect.Signature`''';

param = matr('inspect', 'Parameter')
'''`inspect.Parameter`''';

# %% ../nbs/01_inst.ipynb 20
logger = matr('logging', 'Logger')
'''`logging.Logger`''';

# %% ../nbs/01_inst.ipynb 22
ichain = matr('itertools', 'chain')
'''`itertools.chain`''';

iprod = matr('itertools', 'product')
'''`itertools.product`''';

ziplong = matr('itertools', 'zip_longest')
'''`itertools.zip_longest`''';

# %% ../nbs/01_inst.ipynb 24
dict_keys = matr('_collections_abc', 'dict_keys')
'''`collections.abc.dict_keys`''';

dict_values = matr('_collections_abc', 'dict_values')
'''`collections.abc.dict_values`''';

dict_items = matr('_collections_abc', 'dict_items')
'''`collections.abc.dict_items`''';

# %% ../nbs/01_inst.ipynb 26
moduletype = matr('types', 'ModuleType')
'''`types.ModuleType`''';

wrappertype = matr('types', 'MethodWrapperType')
'''`types.MethodWrapperType`''';

builtinfunctype = matr('types', 'BuiltinFunctionType')
'''`types.BuiltinFunctionType`''';

trackbacktype = matr('types', 'TracebackType')
'''`types.TracebackType`''';

functiontype = matr('types', 'FunctionType')
'''`types.FunctionType`''';

nonetype = matr('types', 'NoneType')
'''`types.NoneType`''';

mapping = matr('collections', 'Mapping')
'''`collections.Mapping`''';

iterable = matr('collections', 'Iterable')
'''`collections.Iterable`''';

nonetype = matr('types', 'NoneType')
'''`types.NoneType`''';

# %% ../nbs/01_inst.ipynb 28
nan = matr('math', 'nan')
'''`math.nan`''';

# %% ../nbs/01_inst.ipynb 29
npnan = matr('numpy', 'nan', default=nan)
'''`numpy.nan`''';

npNaN = matr('numpy', 'NaN', default=nan)
'''`numpy.NaN`''';

npNAN = matr('numpy', 'NAN', default=nan)
'''`numpy.NAN`''';
# npnan, npNaN, npNAN = matrs('numpy', 'nan', 'NaN', 'NAN', default=nan)
pdna = matr('pandas', 'NA', default=nan)
'''`pandas.NA`''';

ptnan = matr('torch', 'nan', default=nan)
'''`torch.nan`''';

# %% ../nbs/01_inst.ipynb 31
number = matr('numbers', 'Number')
'''`numbers.Number`''';

npbool = matr('numpy', 'bool_', default=bool)
'''`numpy.bool_`''';

# %% ../nbs/01_inst.ipynb 33
repattern = matr('re', 'Pattern')
'''`re.Pattern`''';

strtemplate = matr('string', 'Template')
'''`string.Template`''';

# %% ../nbs/01_inst.ipynb 35
nptype = matr('numpy', 'dtype', default=type)
'''`numpy.dtype`''';

npmatrix = matr('numpy', 'matrix', default=list)
'''`numpy.matrix`''';

npdatasource = matr('numpy', 'DataSource', default=pathlike)
'''`numpy.DataSource`''';

nparray = matr('numpy', 'ndarray', default=list)
'''`numpy.ndarray`''';

recarray = matr('numpy', 'recarray', default=list)
'''`numpy.recarray`''';

npempty = matr('numpy', 'empty', default=list)
'''`numpy.empty`''';

# npmatrix, nparray, recarray, npempty = \
#     matrs('numpy', 'matrix', 'ndarray', 'recarray', 'empty', default=list)

# %% ../nbs/01_inst.ipynb 37
sparray = matr('scipy.sparse._arrays', '_sparray')
sparray = matr('scipy.sparse', 'sparray', default=(sparray or list))
'''`scipy.sparse.sparray`''';

spmatrix = matr('scipy.sparse', 'spmatrix', default=list)
'''`scipy.sparse.spmatrix`''';

# %% ../nbs/01_inst.ipynb 38
splinalg = matr('scipy.sparse', 'linalg')
'''`scipy.sparse.linalg`''';

speigs = matr('scipy.sparse.linalg', 'eigs')
'''`scipy.sparse.linalg.eigs`''';

sppdist = matr('scipy.spatial.distance', 'pdist')
'''`scipy.spatial.distance.pdist`''';

# %% ../nbs/01_inst.ipynb 39
cscarray = matr('scipy.sparse', 'csc_array')
'''`scipy.sparse.csc_array`''';

cscmatrix = matr('scipy.sparse', 'csc_matrix')
'''`scipy.sparse.csc_matrix`''';

cooarray = matr('scipy.sparse', 'coo_array')
'''`scipy.sparse.coo_array`''';

coomatrix = matr('scipy.sparse', 'coo_matrix')
'''`scipy.sparse.coo_matrix`''';

csrarray = matr('scipy.sparse', 'csr_array')
'''`scipy.sparse.csr_array`''';

csrmatrix = matr('scipy.sparse', 'csr_matrix')
'''`scipy.sparse.csr_matrix`''';

diaarray = matr('scipy.sparse', 'dia_array')
'''`scipy.sparse.dia_array`''';

diamatrix = matr('scipy.sparse', 'dia_matrix')
'''`scipy.sparse.dia_matrix`''';

dokarray = matr('scipy.sparse', 'dok_array')
'''`scipy.sparse.dok_array`''';

dokmatrix = matr('scipy.sparse', 'dok_matrix')
'''`scipy.sparse.dok_matrix`''';

lilarray = matr('scipy.sparse', 'lil_array')
'''`scipy.sparse.lil_array`''';

lilmatrix = matr('scipy.sparse', 'lil_matrix')
'''`scipy.sparse.lil_matrix`''';

bsrarray = matr('scipy.sparse', 'bsr_array')
'''`scipy.sparse.bsr_array`''';

bsrmatrix = matr('scipy.sparse', 'bsr_matrix')
'''`scipy.sparse.bsr_matrix`''';

# %% ../nbs/01_inst.ipynb 41
pdindex = matr('pandas', 'Index', default=list)
'''`pandas.Index`''';

pdseries = matr('pandas', 'Series', default=list)
'''`pandas.Series`''';

pdcatindex = matr('pandas', 'CategoricalIndex', default=list)
'''`pandas.CategoricalIndex`''';

pddataframe = matr('pandas', 'DataFrame', default=list)
'''`pandas.DataFrame`''';

# pdindex, pdseries, pdcatindex, pddataframe = \
#     matrs('pandas', 'Index', 'Series', 'CategoricalIndex', 'DataFrame', default=list)

# %% ../nbs/01_inst.ipynb 42
pddelegate = matr('pandas.core.accessor', 'PandasDelegate')
'''`pandas.core.accessor.PandasDelegate`''';

pdcataccessor = matr('pandas.core.arrays.categorical', 'CategoricalAccessor')
'''`pandas.core.arrays.categorical.CategoricalAccessor`''';

# %% ../nbs/01_inst.ipynb 44
ptsize = matr('torch', 'size', default=tuple)
'''`torch.Size`''';

pttype = matr('torch', 'dtype', default=type)
'''`torch.dtype`''';

ptdevice = matr('torch', 'device', default=str)
'''`torch.device`''';

# %% ../nbs/01_inst.ipynb 45
ptmodule = matr('torch.nn', 'Module')
'''`torch.nn.Module`''';

ptdataset = matr('torch.utils.data', 'Dataset')
'''`torch.utils.data.Dataset`''';

ptdataloader = matr('torch.utils.data', 'DataLoader')
'''`torch.utils.data.DataLoader`''';
# ptdataset, ptdataloader = matrs('torch.utils.data', 'Dataset', 'DataLoader')

pttensorbase = matr('torch._C', '_TensorBase')
'''`torch._C._TensorBase`''';

pttensor = matr('torch', 'Tensor', bases=(pttensorbase, ))
'''`torch.Tensor`''';

ptinttensor = matr('torch', 'IntTensor', bases=(pttensor, ))
'''`torch.IntTensor`''';

ptbooltensor = matr('torch', 'BoolTensor', bases=(pttensor, ))
'''`torch.BoolTensor`''';

ptfloattensor = matr('torch', 'FloatTensor', bases=(pttensor, ))
'''`torch.FloatTensor`''';

# ptinttensor, ptbooltensor, ptfloattensor = \
#     matrs('torch', 'IntTensor', 'BoolTensor', 'FloatTensor', bases=(pttensor, ))

# %% ../nbs/01_inst.ipynb 46
ptmemfmt = matr('torch', 'memory_format', bases=(type, ))
'''`torch.memory_format`''';

ptpresfmt = matr('torch', 'preserve_format', bases=(ptmemfmt, ))
'''`torch.preserve_format`''';

# %% ../nbs/01_inst.ipynb 48
ptdatasets = matr('torchvision.datasets')
'''`torchvision.datasets`''';

pttransforms = matr('torchvision.transforms')
'''`torchvision.transforms`''';

ptfunctional = matr('torchvision.transforms.functional')
'''`torchvision.transforms.functional`''';

# %% ../nbs/01_inst.ipynb 50
plmodule = matr('pytorch_lightning', 'LightningModule', default=ptmodule)
'''`lightning.LightningModule`''';

pldatamodule = matr('pytorch_lightning', 'LightningDataModule', default=list)
'''`lightning.LightningDataModule`''';

# %% ../nbs/01_inst.ipynb 52
odefn = matr('torchdyn.core.defunc', 'ODEFunc')
'''`torchdyn.core.defunc.ODEFunc`''';

sdefn = matr('torchdyn.core.defunc', 'SDEFunc')
'''`torchdyn.core.defunc.SDEFunc`''';
# odefn, sdefn = matrs('torchdyn.core.defunc', 'ODEFunc', 'SDEFunc')

neurode = matr('torchdyn.core', 'NeuralODE', default=ptmodule)
'''`torchdyn.core.NeuralODE`''';

neursde = matr('torchdyn.core', 'NeuralSDE', default=ptmodule)
'''`torchdyn.core.NeuralSDE`''';
# neurode, neursde = matrs('torchdyn.core', 'NeuralODE', 'NeuralSDE', default=ptmodule)

augmenter = matr('torchdyn.nn', 'Augmenter')
'''`torchdyn.nn.augmenter`''';

# %% ../nbs/01_inst.ipynb 54
ito = matr('torchsde', 'SDEIto')
'''`torchsde.ito`''';

brownian = matr('torchsde', 'BrownianInterval')
'''`torchsde.BrownianInterval`''';

# %% ../nbs/01_inst.ipynb 56
anndata = matr('anndata', 'AnnData')
'''`anndata.AnnData`''';

# %% ../nbs/01_inst.ipynb 58
gtbase = matr('graphtools.base', 'Base')
'''`graphtools.base.Base`''';

gtdata = matr('graphtools.base', 'Data', bases=(gtbase, ))
'''`graphtools.base.Data`''';

bgraph = matr('graphtools.base', 'BaseGraph', bases=(gtbase, ))
'''`graphtools.base.BaseGraph`''';

dgraph = matr('graphtools.base', 'DataGraph', bases=(gtdata, bgraph))
'''`graphtools.base.DataGraph`''';

# %% ../nbs/01_inst.ipynb 59
lgraph = matr('graphtools.graphs', 'LandmarkGraph', bases=(dgraph, ))
'''`graphtools.graphs.LandmarkGraph`''';

kgraph = matr('graphtools.graphs', 'kNNGraph', bases=(dgraph, ))
'''`graphtools.graphs.kNNGraph`''';

tgraph = matr('graphtools.graphs', 'TraditionalGraph', bases=(dgraph, ))
'''`graphtools.graphs.TraditionalGraph`''';

mgraph = matr('graphtools.graphs', 'MNNGraph', bases=(dgraph, ))
'''`graphtools.graphs.MNNGraph`''';

# lgraph, kgraph, tgraph, mgraph = \
#     matrs('graphtools.graphs', 'LandmarkGraph', 'kNNGraph', 'TraditionalGraph', 'MNNGraph', bases=(dgraph, ))

# %% ../nbs/01_inst.ipynb 60
klgraph = matr('graphtools.graphs', 'kNNLandmarkGraph', bases=(kgraph, lgraph, ))
'''`graphtools.graphs.kNNLandmarkGraph`''';

mlgraph = matr('graphtools.graphs', 'MNNLandmarkGraph', bases=(mgraph, lgraph, ))
'''`graphtools.graphs.MNNLandmarkGraph`''';

tlgraph = matr('graphtools.graphs', 'TraditionalLandmarkGraph', bases=(tgraph, lgraph, ))
'''`graphtools.graphs.TraditionalLandmarkGraph`''';

# %% ../nbs/01_inst.ipynb 62
vidcap = matr('cv2', 'VideoCapture')
'''`cv2.VideoCapture`''';

# %% ../nbs/01_inst.ipynb 64
image = matr('PIL.Image', 'Image')
'''`PIL.Image.Image`''';

# %% ../nbs/01_inst.ipynb 66
mpl = matr('matplotlib')
'''`matplotlib`''';

plt = matr('matplotlib', 'pyplot', bases=(ModuleType, ))
'''`matplotlib.pyplot`''';

artist = matr('matplotlib.artist', 'Artist')
'''`matplotlib.artist.Artist`''';

colormap = matr('matplotlib.colors', 'Colormap')
'''`matplotlib.colors.Colormap`''';

lscolormap = matr('matplotlib.colors', 'LinearSegmentedColormap', bases=(colormap, ))
'''`matplotlib.colors.LinearSegmentedColormap`''';

mplpatch = matr('matplotlib.patches', 'Patch', bases=(artist, ))
'''`matplotlib.patches.Patch`''';

mplaxbase = matr('matplotlib.axes._axes', '_AxesBase', bases=(artist, ))
'''`matplotlib.axes._axes._AxesBase`''';

mplaxes = matr('matplotlib.axes._axes', 'Axes', bases=(mplaxbase, ))
'''`matplotlib.axes._axes.Axes`''';

# %% ../nbs/01_inst.ipynb 68
sns = matr('seaborn')
'''`seaborn`''';

# %% ../nbs/01_inst.ipynb 70
skestimator = matr('sklearn.base', 'BaseEstimator')
'''`sklearn.base.BaseEstimator`''';

skpairwise = matr('sklearn.metrics', 'pairwise')
'''`sklearn.metrics.pairwise`''';

# %% ../nbs/01_inst.ipynb 72
PHATE = matr('phate', 'PHATE', bases=(skestimator, ))
'''`phate.PHATE`''';

# %% ../nbs/01_inst.ipynb 74
_jupmix: object = matr('rich.jupyter', 'JupyterMixin')
'''`rich.jupyter.JupyterMixin`''';

# %% ../nbs/01_inst.ipynb 75
richtree: type = matr('rich.tree', 'Tree', bases=(_jupmix, ))
'''`rich.tree.Tree`''';
richtext: type = matr('rich.text', 'Text', bases=(_jupmix, ))
'''`rich.text.Text`''';
richcons: type = matr('rich.console', 'Console', bases=(_jupmix, ))
'''`rich.console.Console`''';
richprog: type = matr('rich.progress', 'Progress', bases=(_jupmix, ))
'''`rich.progress.Progress`''';
richtask: type = matr('rich.progress', 'Task', bases=(_jupmix, ))
'''`rich.tree.Task`''';

# %% ../nbs/01_inst.ipynb 77
_com: object = matr('tqdm.utils', 'Comparable')
'''`tqdm.utils.Comparable`''';
_std: type = matr('tqdm.std', 'tqdm', __bases__=(_com, ))
'''`tqdm.std.tqdm`''';
_t_a: type = matr('tqdm.tqdm_asyncio', 'tqdm_asyncio', __bases__=(_std, ))
'''`tqdm.tqdm_asyncio.tqdm_asyncio`''';
_t_n: type = matr('tqdm.notebook', 'tqdm_notebook', __bases__=(_std, ))
'''`tqdm.notebook.tqdm_notebook`''';

# %% ../nbs/01_inst.ipynb 78
tqdmauto: type = matr('tqdm.auto', 'tqdm', __bases__=(_t_n, _t_a), default=lambda x, desc='': x)
'''`tqdm.auto.tqdm`''';

# %% ../nbs/01_inst.ipynb 80
typer: type = matr('typer', 'Typer')



# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/matr',
                'doc_host': 'https://dsm-72.github.io',
                'git_url': 'https://github.com/dsm-72/matr',
                'lib_path': 'matr'},
  'syms': { 'matr.core': { 'matr.core.ImportedAttribute': ('core.html#importedattribute', 'matr/core.py'),
                           'matr.core.Spec': ('core.html#spec', 'matr/core.py'),
                           'matr.core.Spec.asdict': ('core.html#spec.asdict', 'matr/core.py'),
                           'matr.core.Spec.getvals': ('core.html#spec.getvals', 'matr/core.py'),
                           'matr.core.Spec.setvals': ('core.html#spec.setvals', 'matr/core.py'),
                           'matr.core.iattr': ('core.html#iattr', 'matr/core.py'),
                           'matr.core.iattrs': ('core.html#iattrs', 'matr/core.py'),
                           'matr.core.matr': ('core.html#matr', 'matr/core.py'),
                           'matr.core.matr.__asset_repr__': ('core.html#matr.__asset_repr__', 'matr/core.py'),
                           'matr.core.matr.__call__': ('core.html#matr.__call__', 'matr/core.py'),
                           'matr.core.matr.__getattribute__': ('core.html#matr.__getattribute__', 'matr/core.py'),
                           'matr.core.matr.__getitem__': ('core.html#matr.__getitem__', 'matr/core.py'),
                           'matr.core.matr.__import__': ('core.html#matr.__import__', 'matr/core.py'),
                           'matr.core.matr.__init__': ('core.html#matr.__init__', 'matr/core.py'),
                           'matr.core.matr.__repr__': ('core.html#matr.__repr__', 'matr/core.py'),
                           'matr.core.matr.ib': ('core.html#matr.ib', 'matr/core.py'),
                           'matr.core.matr.io': ('core.html#matr.io', 'matr/core.py'),
                           'matr.core.matrs': ('core.html#matrs', 'matr/core.py'),
                           'matr.core.pmod': ('core.html#pmod', 'matr/core.py'),
                           'matr.core.pmod.__dir__': ('core.html#pmod.__dir__', 'matr/core.py'),
                           'matr.core.pmod.__getattr__': ('core.html#pmod.__getattr__', 'matr/core.py'),
                           'matr.core.pmod.__init__': ('core.html#pmod.__init__', 'matr/core.py'),
                           'matr.core.pmod.__setattr__': ('core.html#pmod.__setattr__', 'matr/core.py'),
                           'matr.core.pmod.attrs': ('core.html#pmod.attrs', 'matr/core.py'),
                           'matr.core.pmod.names': ('core.html#pmod.names', 'matr/core.py'),
                           'matr.core.pmod.populate': ('core.html#pmod.populate', 'matr/core.py')},
            'matr.inst': {}}}
